      subroutine dsddneunuc(sigsip,sigsin,sigsdp,sigsdn)
c_______________________________________________________________________
c  neutralino nucleon cross section.
c  common:
c    'dssusy.h' - file with susy common blocks
c  output:
c    sigsip, sigsin : proton and neutron spin-independent cross sections
c    sigsdp, sigsdn : proton and neutron spin-dependent cross sections
c    units: cm^2
c  author: paolo gondolo (gondolo@lpthe.jussieu.fr) 1994,1995
c     13-sep-94 no drees-nojiri twist-2 terms
c     22-apr-95 important bug corrected [ft -> ft mp/mq]
c=======================================================================
      implicit none
      include 'dssusy.h'
      include 'dsprep.h'
      include 'dsddcom.h'
      include 'dshacom.h'

      real*8 sigsip,sigsdp,sigsin,sigsdn
      integer q,g,i
      real*8 gev2cm2,mp,mq,fkin,f1q,df1q,fp,fn,
     &     du,dd,ds,dsabsq
      ! fp,fn,f1q,df1q,du,dd,ds are twice those in drees-nojiri
      parameter (gev2cm2 = (197.327053d-16)**2)
      parameter (mp=0.93827231d0)

      fkin = 4./pi*(mp*mx/(mp+mx))**2

      fp = 0.
      fn = 0.
      do g=1,3
c     up-quarks
         mq = mass(kqu(g))
         f1q =
     &     gr(kh1,kn(kln),kn(kln))*gl(kh1,kqu(g),kqu(g))/mass(kh1)**2
     &     +gr(kh2,kn(kln),kn(kln))*gl(kh2,kqu(g),kqu(g))/mass(kh2)**2
         do i=1,6
           df1q =
     &        - 0.5*gl(ksqu(i),kn(kln),kqu(g))*
     &              conjg(gr(ksqu(i),kn(kln),kqu(g)))/
     &              (mass(ksqu(i))**2)
c     &        (mass(ksqu(i))**2-(mx+mq)**2)
c     &        + 0.125*(glsnf(i,kln,q)**2+conjg(grsnf(i,kln,q))**2)/
c     &        (mass(ksqu(i))**2-(mx+mq)**2)**2
           f1q = f1q + df1q
         enddo
         q = 5+2*g
         fp = fp + f1q*ftp(q)*(mp/mq)
         fn = fn + f1q*ftn(q)*(mp/mq)
c     down-quarks
         mq = mass(kqd(g))
         f1q =
     &     gr(kh1,kn(kln),kn(kln))*gl(kh1,kqd(g),kqd(g))/mass(kh1)**2
     &     +gr(kh2,kn(kln),kn(kln))*gl(kh2,kqd(g),kqd(g))/mass(kh2)**2
         do i=1,6
           df1q =
     &        - 0.5*gl(ksqd(i),kn(kln),kqd(g))*
     &              conjg(gr(ksqd(i),kn(kln),kqd(g)))/
     &              (mass(ksqd(i))**2)
c     &        (mass(ksqd(i))**2-(mx+mq)**2)
c     &        + 0.125*(glsnf(i,kln,q)**2+conjg(grsnf(i,kln,q))**2)/
c     &        (mass(ksqd(i))**2-(mx+mq)**2)**2
           f1q = f1q + df1q
         enddo
         q = 6+2*g
         fp = fp + f1q*ftp(q)*(mp/mq)
         fn = fn + f1q*ftn(q)*(mp/mq)
      enddo
      sigsip = fkin*(fp/2)**2 ! in gev^-2
      sigsin = fkin*(fn/2)**2 ! in gev^-2

      du = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,ku,ku)-gr(kz,ku,ku))/
     &     mass(kz)**2
      dd = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,kd,kd)-gr(kz,kd,kd))/
     &     mass(kz)**2
      ds = 0.5*gr(kz,kn(kln),kn(kln))*(gl(kz,ks,ks)-gr(kz,ks,ks))/
     &     mass(kz)**2
      do i=1,6
        du = du
     &     + 0.125*(dsabsq(gl(ksqu(i),kn(kln),ku))+
     &              dsabsq(gr(ksqu(i),kn(kln),ku)))/
     &      (mass(ksqu(i))**2)
c     &     (mass(ksqu(i))**2-(mx+mass(ku))**2)
        dd = dd
     &     + 0.125*(dsabsq(gl(ksqd(i),kn(kln),kd))+
     &              dsabsq(gr(ksqd(i),kn(kln),kd)))/
     &      (mass(ksqd(i))**2)
c     &     (mass(ksqd(i))**2-(mx+mass(kd))**2)
        ds = ds
     &     + 0.125*(dsabsq(gl(ksqd(i),kn(kln),ks))+
     &              dsabsq(gr(ksqd(i),kn(kln),ks)))/
     &      (mass(ksqu(i))**2)
c     &     (mass(ksqu(i))**2-(mx+mass(ks))**2)
      enddo
      sigsdp = fkin*(3./4.)*(du*delu+dd*deld+ds*dels)**2 ! in gev^-2
      sigsdn = fkin*(3./4.)*(du*deld+dd*delu+ds*dels)**2 ! in gev^-2

      sigsip = gev2cm2*sigsip
      sigsin = gev2cm2*sigsin
      sigsdp = gev2cm2*sigsdp
      sigsdn = gev2cm2*sigsdn

      return
      end
