**********************************************************************
*** subroutine dsddrates calculates the rates in direct detection
*** epxeriments. results are given for ge, alo and nai. the cross
*** sections are also given as output for reference.
*** routines written by l. bergstrom and p. gondolo
*** date: 1995
**********************************************************************

      subroutine dsddrates(rate,sigma)
      implicit none

      include 'dssusy.h'
      include 'dsidtag.h'
      real*8 amu,mp,n_a,
     &    sigsip,sigsin,sigsdp,sigsdn,ratege,ratealo,ratenai,
     &    sigdirac,mx,fkin,anuc,mnuc,
     &    dsddu_eff,ueff,ueffbis,lamjj,nkg,signuc,vesc
      real*8 rate(3),sigma(4)
      parameter (amu=0.93149432d0)
      parameter (mp=0.93827231d0)
      parameter (n_a=6.0221367d23)
      parameter (vesc=11.9d0)
      include 'dshmcom.h'

c===================================================elastic off nucleons

      call dsddneunuc(sigsip,sigsin,sigsdp,sigsdn)

c----------------------------------------normalization to dirac neutrino

      mx = abs(mass(kn(kln)))
      fkin = (mx*mp/(mx+mp))**2
      sigdirac = (1.16639d-5*197.327)**2/(8.d0*pi) * fkin * 1d-32 ! in cm^2

c=================================================direct detection rates

c-----------------------------------------------------------germanium-76
      anuc = 76.                ! atomic weight of target nucleus
      mnuc = anuc*amu
      fkin = ( (mx*mnuc/(mnuc+mx)) / (mx*mp/(mp+mx)) )**2
      signuc = anuc**2 * fkin * sigsip ! in cm^2
      nkg = n_a * (1000./anuc)
      ueff = dsddu_eff(anuc,mx,vesc,vobs,vdisp,vgalesc,1,0) ! in km/sec
      ueffbis = 86400. * 1.d5 * ueff ! in cm/day
      ratege = nkg * (rhox/mx) * signuc * ueffbis ! in ev/kg/day

c------------------------------------------------------zaphyr (al_2 o_3)
c   aluminum
      anuc = 27.                ! atomic weight of target nucleus
      ! lamjj = 0.35            ! ellis-flores naive shell model
      lamjj = 0.087             ! engel-vogel odd-group model
      mnuc = anuc*amu
      fkin = ( (mx*mnuc/(mnuc+mx)) / (mx*mp/(mp+mx)) )**2
      signuc = fkin * (anuc**2*sigsip + (4*lamjj/3)*sigsdp) ! in cm^2
      ueff = dsddu_eff(anuc,mx,vesc,vobs,vdisp,vgalesc,1,0) ! in km/sec
      ueffbis = 86400. * 1.d5 * ueff ! in cm/day
      nkg = (1000./27.) * n_a * (2*27./(2*27.+3*16.))
      ratealo = nkg * (rhox/mx) * signuc * ueffbis ! in ev/kg/day
c   oxygen
      anuc = 16.                ! atomic weight of target nucleus
      mnuc = anuc*amu
      fkin = ( (mx*mnuc/(mnuc+mx)) / (mx*mp/(mp+mx)) )**2
      signuc = anuc**2 * fkin * sigsip ! in cm^2
      ueff = dsddu_eff(anuc,mx,vesc,vobs,vdisp,vgalesc,1,0) ! in km/sec
      ueffbis = 86400. * 1.d5 * ueff ! in cm/day
      nkg = (1000./16.) * n_a * (3*16./(2*27.+3*16.))
      ratealo = ratealo + nkg * (rhox/mx) * signuc * ueffbis ! in ev/kg/day

c----------------------------------------------------sodium iodide (nai)
c   sodium
      anuc = 23.                ! atomic weight of target nucleus
      ! lamjj = 0.35            ! ellis-flores naive shell model
      lamjj = 0.041             ! engel-vogel odd-group model
      mnuc = anuc*amu
      fkin = ( (mx*mnuc/(mnuc+mx)) / (mx*mp/(mp+mx)) )**2
      signuc = fkin * (anuc**2*sigsip + (4*lamjj/3)*sigsdp) ! in cm^2
      ueff = dsddu_eff(anuc,mx,vesc,vobs,vdisp,vgalesc,1,0) ! in km/sec
      ueffbis = 86400. * 1.d5 * ueff ! in cm/day
      nkg = (1000./23.) * n_a * (23./(23.+117.))
      ratenai = nkg * (rhox/mx) * signuc * ueffbis ! in ev/kg/day
c   iodine
      anuc = 117.                ! atomic weight of target nucleus
      ! lamjj = 0.25             ! ellis-flores naive shell model
      lamjj = 0.007              ! engel-vogel odd-group model
      mnuc = anuc*amu
      fkin = ( (mx*mnuc/(mnuc+mx)) / (mx*mp/(mp+mx)) )**2
      signuc = fkin * (anuc**2*sigsip + (4*lamjj/3)*sigsdp) ! in cm^2
      ueff = dsddu_eff(anuc,mx,vesc,vobs,vdisp,vgalesc,1,0) ! in km/sec
      ueffbis = 86400. * 1.d5 * ueff ! in cm/day
      nkg = (1000./117.) * n_a * (117./(23.+117.))
      ratenai = ratenai + nkg * (rhox/mx) * signuc * ueffbis ! in ev/kg/day

c----------------------------------------------------- results in array

      sigma(1)=sigsip
      sigma(2)=sigsin
      sigma(3)=sigsdp
      sigma(4)=sigsdn

      rate(1)=ratege
      rate(2)=ratealo
      rate(3)=ratenai

      return
      end










