      function dsddu_inverse(u1,u2,vobs,sigma,vcut)
c
c     the average <1/u> = \integral_{u1}^{u2} {du f(u) \over u}
c     with f(u) a truncated maxwellian with 1-d dispersion sigma
c     seen by an observer at velocity vobs
c
      implicit none
      real*8 dsddu_inverse,u1,u2,vobs,sigma,vcut
      real*8 x1,x2,eta,etacut,xcut,norm,sqrt2,erf,
     &   eta2,etacut2,auxsig,twovrsqrtpi,bracket
      parameter (sqrt2=1.41421356237d0)
      parameter (twovrsqrtpi=1.12837916709551d0)
      dsddu_inverse=0.0
      if (u2.ge.vobs+vcut) u2=vobs+vcut
      if (u1.ge.u2) return
      auxsig=sqrt2*sigma
      x1=u1/auxsig
      x2=u2/auxsig
      eta=vobs/auxsig
      eta2=eta*eta
      etacut=vcut/auxsig
      etacut2=etacut*etacut
      norm=erf(etacut)-twovrsqrtpi*etacut*exp(-etacut2)
      xcut=etacut-eta
      ! case a: u1 < u2 < ucut
      if (x2.le.xcut .or. vcut.gt.300000.) then
	bracket=erf(x2-eta)-erf(x2+eta)-erf(x1-eta)+erf(x1+eta)
      ! case c: ucut < u1 < u2
      else if (x1.ge.xcut) then
	bracket=exp(-2*eta2+etacut2)/sqrt2*
     &     (erf(sqrt2*x2)-erf(sqrt2*x1)) -
     &     (erf(x2+eta)-erf(x1+eta))
      ! case b: u1 < ucut < u2
      else
	bracket=erf(xcut-eta)-erf(xcut+eta)-
     &     erf(x1-eta)+erf(x1+eta)+
     &     exp(-2*eta2+etacut2)/sqrt2*
     &     (erf(sqrt2*x2)-erf(sqrt2*xcut)) -
     &     (erf(x2+eta)-erf(xcut+eta))
      endif
      dsddu_inverse=bracket/(2*norm*vobs)
      return
      end
