**********************************************************************
*** function dsepdiff calculates the differential flux of
*** positrons for the energy egev as a result of
*** neutralino annihilation in the halo.
*** input: egev - positron energy in gev
***        how = 1 - dsepsigvdnde is used directly
***              2 - dsepsigvdnde is tabulated on first call, and then
***                  interpolated.
*** units: gev^-1 cm^-2 sec^-1 sr^-1
*** author: e.a. baltz (eabaltz@astron.berkeley.edu)
***         joakim edsjo, edsjo@physto.se
*** date: jun-02-98
*** modified: 99-07-02 paolo gondolo : order of calls to hrsetup
**********************************************************************

      real*8 function dsepdiff(egev,how)
      implicit none

      include 'dsidtag.h'
      include 'dshacom.h'
      include 'dsepcom.h'
      include 'dssusy.h'
      include 'dshmcom.h'
      include 'dsprep.h'

c----------------------------------------------------------------------

      real*8 egev,sigvline,dsepsigvdnde,dsepspec,dsepipol
      real*8 dsepeeuncut,dsepeecut,dsepvvuncut,dsepvvcut,
     &  dsepwuncut,dsepwcut,dsepktdiff
      integer how
      external dsepsigvdnde,dsepipol,dsepeeuncut,dsepeecut,
     &  dsepvvuncut,dsepvvcut,dsepwuncut,dsepwcut

c----------------------------------------------------------------------

      if (.not.dsprepcalled) then
        write(*,*) 'error in dsepdiff: dsprep must be called',
     &    ' before any rate calculation'
        write(*,*) 'begins for every new model. program stopping.'
      endif
 
      sigvline=sigv(15)  ! cm^-3 s^-1

      if (dsepdiffmethod.eq.2) then ! kamionkowski & turner propagation
        dsepdiff=dsepktdiff(egev)
        return
      endif

      if (newmodelep.and.how.eq.2) then ! new model
         call dseptab(0.1d0,200)
         newmodelep=.false.
      endif

c...  setup common block for integration

      if (how.eq.1.and.dsepdiffmethod.eq.0) then
         dsepdiff=dsepspec(egev,mx,sigvline,dsepsigvdnde,
     &     dsepeeuncut,dsepvvuncut,dsepwuncut)
      else if (how.eq.1.and.dsepdiffmethod.eq.1) then
         dsepdiff=dsepspec(egev,mx,sigvline,dsepsigvdnde,
     &     dsepeecut,dsepvvcut,dsepwcut)
      else if (how.eq.2.and.dsepdiffmethod.eq.0) then
         dsepdiff=dsepspec(egev,mx,sigvline,dsepipol,dsepeeuncut,
     &     dsepvvuncut,dsepwuncut)
      else
         dsepdiff=dsepspec(egev,mx,sigvline,dsepipol,dsepeecut,
     &     dsepvvcut,dsepwcut)
      endif

c...Now rescale
      dsepdiff=(rhox/rho0)**2*dsepdiff

      return
      end

