**********************************************************************
*** function dsepktline calculates the flux of e+ from the line
*** annihilation.
*** from kamionkowski & turner, prd 43(1991)1774.
*** units: gev^-1 cm^-2 sec^-1 sr^-1
*** author: joakim edsjo, edsjo@physto.se
*** date: 98-02-10
**********************************************************************

      real*8 function dsepktline(egev)
      implicit none

      include 'dssusy.h'
      include 'dshmcom.h'
      include 'dshrcom.h'
      include 'dshacom.h'
      include 'dsidtag.h'
      include 'dsprep.h'

c----------------------------------------------------------------------

      real*8 egev,q,tcont,yr2sec,hc2,
     &  gfunc,tau7,r,brepem
      parameter (yr2sec=3.15576d7)
      parameter (hc2=0.38937966d-27)  ! gev^2 cm^2
      parameter (tau7=1.0d0) ! yr
c----------------------------------------------------------------------

      if (.not.dsprepcalled) then
        write(*,*) 'error in dsepktline: dsprep must be called',
     &    ' before any rate calculation'
        write(*,*) 'begins for every new model. program stopping.'
      endif

      if (egev.le.mass(kn(1))) then   ! change to mx?
cpg        sigv = 0.38937966d-27*3.d10*wtot/(2.0d0*mx**2) ! in cm^3/s
        brepem=sigv(15)/sigmav  ! br to e+ e-   wtot -> sigmav pg990223
c        brepem=0.01d0  ! delete later

c...source strength
        q=(rhox/mx)**2*sigmav/(4.0d0*pi)  ! cm^-3 s^-1

c...containment time
        tcont=1d7*tau7*yr2sec*20.0/mx  ! containment time, sec

c...greens function
        r=66.0d0/(tau7*20.0d0)
        gfunc=6.0d26*q/mx**2*(egev/mx)**(r-2.0d0)

c        gfunc=6.0d26*q/egev**2*(egev/mx)**(r-2.0d0) ! just for testing

        dsepktline=gfunc*brepem

      else
        dsepktline=0.0d0
      endif

      return
      end

