      subroutine dshmset(c)
****************************************************************
*** subroutine dshmset:                                      ***
*** initialize the density profile dshmhaloprof.f and define ***
*** the type of halo:                                        ***
***   hclumpy=1 smooth, hclumpy=2 clumpy                     ***
***                                                          ***
*** a few sample cases are given; specified values of the    ***
*** local halo density 'rho0' and of the length scale        ***
*** parameter 'a' should be considered just indicative       ***
***                                                          ***
*** author: piero ullio (piero@tapir.caltech.edu)            ***
*** date: 00-07-13                                           ***
*** small modif: paolo gondolo 00-07-19                      ***
****************************************************************
      implicit none
      include 'dshmcom.h'
      character*(*) c

c the velocity distribution is assumed maxwellian even if
c      this is inconsistent with some density profiles (pg)
      vdisp = 120.d0            ! wimp 1-dim velocity dispersion in km/s
      vgalesc = 600.d0          ! galactic escape speed in km/s
      vobs = 264.d0             ! observer speed w.r.t. the halo in km/s

c a few options for the density profile:

c ... modified isothermal profile, smooth profile
      if (c.eq.'isosm'.or.c.eq.'default') then
        hclumpy=1
        r_0=8.5d0           !sun galactocentric distance (kpc)
        rho0=0.3d0          !local halo density (gev cm^-3)  
        a=3.5d0             !length scale (kpc)
        alphah=2.d0
        beta=2.d0
        gamma=0.d0


c ... modified isothermal profile, clumpy profile
      elseif (c.eq.'isocl') then
        hclumpy=2
        r_0=8.5d0           !sun galactocentric distance (kpc)
        rho0=0.3d0          !local halo density (gev cm^-3)  
        a=5.d0              !length scale (kpc)
        alphah=2.d0
        beta=2.d0
        gamma=0.d0


c ... navarro-frenk-white profile, smooth profile
      elseif (c.eq.'nfwsm') then
        hclumpy=1
        r_0=8.d0            !sun galactocentric distance (kpc)
        rho0=0.3d0          !local halo density (gev cm^-3)  
        a=20.d0             !length scale (kpc)
        alphah=1.d0
        beta=3.d0
        gamma=1.d0


c ... moore et al. profile, smooth profile
      elseif (c.eq.'moosm') then
        hclumpy=1
        r_0=8.d0            !sun galactocentric distance (kpc)
        rho0=0.3d0          !local halo density (gev cm^-3)  
        a=28.d0             !length scale (kpc)
        alphah=1.5d0
        beta=3.d0
        gamma=1.5d0


c ... kravtsov et al. profile, smooth profile
      elseif (c.eq.'krasm') then
        hclumpy=1
        r_0=8.d0            !sun galactocentric distance (kpc)
        rho0=0.6d0          !local halo density (gev cm^-3)  
        a=10.d0             !length scale (kpc)
        alphah=2.d0
        beta=3.d0
        gamma=0.4d0

c...invalid choice
      else
         write (*,*) 'dshmset: unrecognized option ',c
         stop
      endif

c...unrescaled local density
      rhox = rho0

      return
      end
