**********************************************************************
*** function dshrgacsusy gives the susy dependent term in the
*** flux of gamma-rays with continuum energy spectrum above the 
*** threshold egath (gev) from neutralino annihilation in the halo.
***
*** dshrgacsusy is dimensionless
*** 
*** the flux in a solid angle delta in the direction psi0 is given by:
***   cm^-2 s^-1 sr^-1 * dshrgacsusy(egath,istat)
***   * dshmjave(cospsi0,delta) * delta (in sr)
***
*** the flux per solid angle in the direction psi0 is given by:
***   cm^-2 s^-1 sr^-1 * dshrgacsusy(egath,istat)
***   * dshmj(cospsi0)
*** 
*** in case of a clumpy halo the factor fdelta has to be added 
***
*** author: joakim edsjo, edsjo@physto.se
*** modified: piero ullio (piero@tapir.caltech.edu)
*** date: 00-07-13
**********************************************************************

      real*8 function dshrgacsusy(egath,istat)
      implicit none
      include 'dssusy.h'
      include 'dshmcom.h'
      include 'dshacom.h'
      include 'dsprep.h'
      real*8 phiga,dshaloyield,egath,dnsigma
      integer istat
c...pg dshaloyield calls hrsetup which assigns sigmav and mx in dshacom.h
      phiga=dshaloyield(egath,52,istat)
      dnsigma=phiga*(sigmav/1.0d-29)
      dshrgacsusy=1.87d-11*dnsigma*(10.0d0/mx)**2
      end
