      real*8 function dshrpbdiff0(tp,solarmod,how)

**********************************************************************
*** function dshrpbdiff0 calculates the differential flux of
*** antiprotons for the antiproton kinetic energy tp as a result of
*** neutralino annihilation in the halo.
*** dshrpbdiff0 uses the unrescaled local density
*** see dshrpbardiff for rescaling the local density
***   input:
***     tp - antiproton kinetic energy in gev
***     solarmod - 0 no solar modulation
***                1 solar modulation a la perko
***     how - 1 calculate t_diff only for requested momentum
***           2 tabulate t_diff for first call and use table for
***             subsequent calls
***           3 as 2, but also write the table to disk as pbtd.dat
***             at the first call
***           4 read table from disk on first call, and use that for
***             subsequent calls
*** units: gev^-1 cm^-2 s^-1 sr^-1
*** author: joakim edsjo
*** date: 98-02-10
*** modified: joakim edsjo, edsjo@physto.se
*** modified: 98-07-13, 00-07-19 paolo gondolo
**********************************************************************

      implicit none

      include 'dshmcom.h'
      include 'dshacom.h'
      include 'dsprep.h'

c----------------------------------------------------------------------
      real*8 tp,tpb,dqde,mp,pi,vtdiff,
     &  difftime,pp,epb,ppb,dshaloyield,phiap,dspbtd15,dspbtpb
      integer istat,solarmod,how
      parameter (mp=0.93827231d0,pi=3.141592653589793238d0)

c----------------------------------------------------------------------

c...solar modulation: calculate kinetic energy at heliosphere boundary
c...antiproton flux before modulation at kinetic energy tpb
      if (solarmod.eq.1) then  ! solar modulation
        tpb=dspbtpb(tp)
        pp=dsqrt(2*mp*tp+tp**2)
        ppb=dsqrt(2*mp*tpb+tpb**2)
        epb=mp+tpb
        phiap=dshaloyield(tpb,154,istat)*pp**2/ppb**2
      else               ! no solar modulation
        tpb=tp
        pp=dsqrt(2*mp*tp+tp**2)
        ppb=pp
        epb=mp+tpb
        phiap=dshaloyield(tp,154,istat)
      endif

c...antiproton flux after modulation
      dqde = (rho0/mx)**2*sigmav*phiap/(4.0d0*pi)
      difftime = dspbtd15(tpb,how)
      vtdiff = (ppb/epb*2.99792458d10)*difftime*1.d15
      dshrpbdiff0 = vtdiff*dqde

      return
      end
