*****************************************************************************
*** subroutine dsmuinit initializes and loads (from disk) the common
*** block variables needed by the other muon yield routines.
*** flxk is the yield type (1,2 or 3 (or 101, 102, 103)) for neutrino yields
*** muon distributions at creation or muon yields at a detector respectively.
*** flxk is used to check that the provided data file is of the correct type.
*** if flxk=1,2 or 3 integrated yields are loaded and if flxk=101, 102 or
*** 103, differential yields are loaded
*** author: joakim edsjo  edsjo@physto.se
*** date: 96-10-23
*** modified: 97-12-03
*****************************************************************************

      subroutine dsmuinit(flxk)
      implicit none
      include 'dsmucom.h'
      include 'dsprep.h'
      include 'dsio.h'

c------------------------ variables ------------------------------------

      integer i,j,k,l,m,flxk,flxkfile,fl
      character*150 filein
      character*255 scr
      character*10 scr2
      character*15 filepref
      character*3 filenr
      character*10 filesuf

c----------------------------------------- set-up common block variables

c... number of bins in tables
c... the actual table in the file covers 0:n-1 but the variable in
c... memory is bigger to make interpolation at edges simple
      thn=90  ! number of bins in theta
      zn=50   ! number of bins in z

c...masses of annihilation products c, b, t, tau, w and z.
      map(1)=1.35
      map(2)=5.0
      map(3)=175.0
      map(4)=1.7841
      map(5)=80.25
      map(6)=91.2

c...lower and upper bounds on neutralino masses for which different
c...annihilation channels can be used
      lb(1)=10.0
      lb(2)=10.0
      lb(3)=176.0
      lb(4)=10.0
      lb(5)=80.3
      lb(6)=91.2
      do i=1,6
        ub(i)=5000.0
      enddo

c...masses for simulation corresponding to mass index i
      mi(1)=10.0
      mi(2)=25.0
      mi(3)=50.0
      mi(4)=80.3
      mi(5)=91.2
      mi(6)=100.0
      mi(7)=150.0
      mi(8)=176.0
      mi(9)=200.0
      mi(10)=250.0
      mi(11)=350.0
      mi(12)=500.0
      mi(13)=750.0
      mi(14)=1000.0
      mi(15)=1500.0
      mi(16)=2000.0
      mi(17)=3000.0
      mi(18)=5000.0

c...lowest mass index for channel j
      milow(1)=1   ! c c-bar
      milow(2)=1   ! b b-bar
      milow(3)=8   ! t t-bar
      milow(4)=1   ! tau+ tau-
      milow(5)=4   ! w+ w-
      milow(6)=5   ! z z

c...initialize thindex array where the thetas for the bins are stored
c...integrated yields (upper end of each bin)
      do i=-1,49
        thindex(i,1)=dble(i+1)*0.2d0
      enddo
      do i=50,90
        thindex(i,1)=dble(i-30+1)*0.5d0
      enddo

c      do i=-1,thn
c        thindex(i,1)=dble(i+1)*0.5d0
c      enddo

c...differential yields (center of each bin)
      do i=-1,49
        thindex(i,2)=dble(i)*0.2d0+0.1d0
        dth(i)=0.2d0
      enddo
      do i=50,90
        thindex(i,2)=dble(i-30)*0.5d0+0.25d0
        dth(i)=0.5d0
      enddo

c      do i=-1,thn
c        thindex(i,2)=dble(i)*0.5d0+0.25d0
c        dth(i)=0.5d0
c      enddo


c...initialize zindex array where the energies for the bins are stored
c...integrated yields (lower end of each bin)
      do i=-1,zn
        zindex(i,1)=dble(i)*0.02d0
      enddo

c...differential yields (center of each bin)
      do i=-1,zn
        zindex(i,2)=dble(i)*0.02d0+0.01d0
        dz(i)=0.02d0
      enddo

c------------------------------------------------------ load muon tables

c...generate file name
      if (mfftype.eq.'a'.or.mfftype.eq.'a') then
        filesuf='.dat'
      else
        filesuf='.bin'
      endif
      if (flxk.lt.100) then   ! integrated
        filepref='sim'//mfmed//'int'
        write(filenr,'(i3)') flxk
      else                    ! differential
        filepref='sim'//mfmed//'diff'
        write(filenr,'(i3)') flxk
      endif
      filein=mfdir//filepref//filenr//filesuf

c...delete spaces in file name
      fl=150
      do l=1,fl
 40     if (filein(l:l).eq.' ') then
          fl=fl-1
          do m=l,fl
            filein(m:m)=filein(m+1:m+1)
          enddo
          if (fl.eq.l) goto 50
          goto 40
        endif
      enddo
 50   continue


      if (flxk.lt.100) then
c...integrated yields
c        write(*,*) 'enter file name for integrated muon yield tables:'
c        read(*,'(a)') filein
c        write(*,*) filein
c        write(*,*) 'is this file ascii (a) or binary (b)?'
c        read(*,'(a)') filetype
c        write(*,*) filetype
        do i=1,2
          do j=1,6
            do k=1,18
              do l=0,zn
                do m=-1,thn
                  phiint(m,l,k,j,i)=0.0
                enddo
              enddo
            enddo
          enddo
        enddo
        if (prtlevel.gt.1) write(*,*)
     &       'loading integrated muon yield tables from file ',filein
        if (mfftype.eq.'a'.or.mfftype.eq.'a') then
          open(unit=13,file=filein,status='old',form='formatted',
     &          err=100)
          read(13,500) scr2,flxkfile
          read(13,'(a)') scr
        else
          open(unit=13,file=filein,status='old',form='unformatted',
     &          err=100)
          read(13) flxkfile
        endif
 500    format(1x,a10,1x,i3)

        if (flxk.ne.flxkfile.and.prtlevel.gt.0) then
          write(*,*) 'error in muflux_init: '
          write(*,*) 'the requested yield type is inconsistent with',
     &      ' the provided data file.'
          write(*,*) 'data file: ',filein
          write(*,*) 'requested fluxtype:   ',flxk
          write(*,*) 'data file''s fluxtype: ',flxkfile
          write(*,*) 'program stopped.'
          stop
        endif

        flxtype(1)=flxkfile
        do i=1,2
          if (i.eq.1) then
             if (prtlevel.gt.1) write(*,*) '   ...for the sun...'
          else
             if (prtlevel.gt.1) write(*,*) '   ...for the earth...'
          endif
          do j=1,6
             if (prtlevel.gt.1) write(*,*) '      channel number ',j
            do k=1,18
              if (k.ge.milow(j)) then
                do l=0,zn-1
                  if (mfftype.eq.'a'.or.mfftype.eq.'a') then
                    read(13,2000) (phiint(m,l,k,j,i),m=0,thn)
                  else
                    read(13) (phiint(m,l,k,j,i),m=0,thn)
                  endif
                enddo
                do l=0,zn-1
                  phiint(-1,l,k,j,i)=0.0
                enddo
              endif
            enddo
          enddo
        enddo
        close(13)

      else   ! differential yields

c...differential yields
c        write(*,*) 'enter file name for differential muon yield tables:'
c        read(*,'(a)') filein
c        write(*,*) filein
c        write(*,*) 'is this file ascii (a) or binary (b)?'
c        read(*,*) filetype
c        write(*,*) filetype
        do i=1,2
          do j=1,6
            do k=1,18
              do l=-1,zn
                do m=-1,thn
                  phidiff(m,l,k,j,i)=0.0
                enddo
              enddo
            enddo
          enddo
        enddo
        if (prtlevel.gt.1) write(*,*)
     &       'loading differential muon yield tables from file ',filein
        if (mfftype.eq.'a'.or.mfftype.eq.'a') then
          open(unit=13,file=filein,status='old',form='formatted',
     &          err=100)
          read(13,500) scr2,flxkfile
          read(13,'(a)') scr
        else
          open(unit=13,file=filein,status='old',form='unformatted',
     &          err=100)
          read(13) flxkfile
        endif
        if (flxk.ne.flxkfile.and.prtlevel.gt.0) then
          write(*,*) 'error in muflux_init: '
          write(*,*) 'the requested yield type is inconsistent with',
     &      ' the provided data file.'
          write(*,*) 'data file: ',filein
          write(*,*) 'requested fluxtype:   ',flxk
          write(*,*) 'data file''s fluxtype: ',flxkfile
          write(*,*) 'program stopped.'
          stop
        endif

        flxtype(2)=flxkfile
        do i=1,2
          if (i.eq.1) then
            if (prtlevel.gt.1) write(*,*) '   ...for the sun...'
          else
            if (prtlevel.gt.1) write(*,*) '   ...for the earth...'
          endif
          do j=1,6
            if (prtlevel.gt.1) write(*,*) '      channel number ',j
            do k=1,18
              if (k.ge.milow(j)) then
                do l=0,zn-1
                  if (mfftype.eq.'a'.or.mfftype.eq.'a') then
                    read(13,2000) (phidiff(m,l,k,j,i),m=0,thn)
                  else
                    read(13) (phidiff(m,l,k,j,i),m=0,thn)
                  endif
                  do m=0,thn ! correct units of dyield / dz dtheta
                    phidiff(m,l,k,j,i)=phidiff(m,l,k,j,i)/
     &                (dth(m)*dz(l))
                  enddo
                enddo
                phidiff(-1,-1,k,j,i)=0.0
                do l=-1,zn
c                  phidiff(-1,l,k,j,i)=phidiff(0,l,k,j,i)
                  phidiff(-1,l,k,j,i)=-4.0/3.0*phidiff(0,l,k,j,i) ! je 961023
                  phidiff(0,l,k,j,i)=4.0/3.0*phidiff(0,l,k,j,i)   ! je 961023
                enddo
                do m=-1,thn
                  phidiff(m,-1,k,j,i)=phidiff(m,0,k,j,i)
                  phidiff(m,zn,k,j,i)=phidiff(m,zn-1,k,j,i)
                enddo
              endif
            enddo
          enddo
        enddo
        close(13)

      endif

      if (prtlevel.gt.1) write(*,*) 
     &    'loading of muon yield tables finished.'

 2000 format(91(1x,e12.6))

      return

 100  continue
      write(*,*) 'error in muflux_init: while opening file ',filein
      write(*,*) 'program stopped.'
      stop
      end
