

***********************************************************************
*** dsntearthdenscomp gives the number density of nucleons of mass
*** number a per cm^3
*** input: radius - in meters
***        mass number - 16 for o
***                      24 for mg
***                      28 for si
***                      56 for fe
*** the radius should be given in m and the density is returned in
*** g/cm^3
*** author: joakim edsjo
*** date: april 6, 1999
***********************************************************************

      real*8 function dsntearthdenscomp(r,a)
      implicit none
      real*8 r,n_a,m_a,dsntearthdens,rcore,fraction
      integer a,index
      parameter (n_a=6.022d23,    ! number per mol
     &  rcore=3488.0d3)           ! division between core and mantle

      real*8 mfrac(2,4) ! core and mantle
      data mfrac/0.05d0,0.4366d0,   ! o
     &           0.0d0,0.2304d0,    ! mg
     &           0.0d0,0.2100d0,    ! si
     &           0.90d0,0.0591d0/   ! fe

      if (a.eq.16) then ! o
        index=1
        m_a=16.0d0
      elseif (a.eq.24) then ! mg
        index=2
        m_a=24.3d0
      elseif (a.eq.28) then ! si
        index=3
        m_a=28.1d0
      elseif (a.eq.56) then ! fe
        index=4
        m_a=55.85d0
      else
        write(*,*) 'warning in dsntearthdenscomp:'
        write(*,*) '  element ',a,' requested - not available!'
        write(*,*) '  please update dsntearthdenscomp'
        dsntearthdenscomp=0.0d0
        return
      endif

c...convert m_a to gev
      m_a=m_a*0.94d0

      if (r.lt.rcore) then
        fraction=mfrac(1,index)
      else
        fraction=mfrac(2,index)
      endif

      dsntearthdenscomp=fraction*dsntearthdens(r)/m_a*n_a ! number/cm^3

      return
      end
