**********************************************************************
*** function called in dspbtd15x
*** it gives the antiproton diffusion time in units of 10^15 sec
*** it assumes the diffusion model in:
***   bergstrom, edsjo & ullio, ajp 526 (1999) 215
*** inputs:
***     tp - antiproton kinetic energy (gev)
*** 
*** author: piero ullio (piero@tapir.caltech.edu)
*** date: 00-07-13
**********************************************************************

      real*8 function dspbtd15beu(tp)
      implicit none
      include 'dspbcom.h'
      include 'dspbprivate.h'
      integer i
      real*8 tp,pp,ee,mp,kpc,cc,partial,total,dspbkdiff,dspbsigmavpbar
      real*8 abserr,alist,blist,elist,epsabs,epsrel,rlist,
     & result,dbesj0,dbesj1
      integer ier,iord,last,limit,neval
      dimension alist(5000),blist(5000),elist(5000),iord(5000),
     & rlist(5000)
      external dspbbeuparm
      parameter (mp=0.93827231d0)
      parameter (kpc=3.08567802d0,cc=2.99792458d0)
      epsabs=1.d-10     !numerical accuracy
      epsrel=1.d-10
      limit=5000
      pp=dsqrt(2*mp*tp+tp**2)
      ee=tp+mp
      dh=dspbkdiff(pp,1)
      dg=dspbkdiff(pp,2)
      axsec=dspbsigmavpbar(ee)
      partial=0.d0
      total=0.d0
      do i=1,pbnzero
        zero=zerovec(i)
c  lambdag,gamh,lambdah in 10**(-21) cm^-1
        lambdag=dsqrt(axsec*pbng/dg/100.d0+(zero/pbrh/kpc)**2)
        gamh=pbcvel/2.d0/dh/10.d0
        lambdah=dsqrt(gamh**2+axsec*pbnh/dh/100.d0
     &                +(zero/pbrh/kpc)**2)
        call dqagse(dspbbeuparm,0.d0,1.d0,epsabs,epsrel,limit,result,
     &      abserr,neval,ier,alist,blist,rlist,elist,iord,last)
        partial=result*dbesj0(zero*pbr0/pbrh)/(dbesj1(zero))**2
     &       *2.d0/(pbrh*kpc)**2
        total=total+partial
      enddo
c  dspbtd15beu in 10^15 sec 
      dspbtd15beu = total
      return
      end







