      real*8 function dsrdomega(omtype,fast,xf,ierr,iwar,nfc)

**********************************************************************
*** function dsrdomega calculates omega h^2 for the mssm neutralino
*** uses the mssm routines and the relic density routines
*** input:
***   omtype = 0 - no coann
***            1 - coann (calc without if not relevant)
***            2 - coann (calc omega only if coann are relevant)
***            3 - quick and dirty method, no coann
***   fast =   0 - standard accurate calculation (accuracy better than 1%)
***            1 - faster calculation:
***                * optimizes annihilation rate
***                * requires less accuracy in tabulation of w_eff
*** output:
***   ierr = error from dsrdens or dsrdqad
*** authors: joakim edsjo, paolo gondolo
*** date: 98-03-03
*** modified: 98-03-03
***           99-07-30 pg
**********************************************************************

      implicit none

      include 'dssusy.h'
      include 'dsidtag.h'
      include 'dsandwcom.h'
      include 'dsrdcom.h'

      real*8 oh2,mcoann(20),dof(20),dsanwx,tm(20),rm(20),rw(20),xf,
     &     tf,h1width
      integer ncoann,ierr,iwar,nt,nrs,omtype,nfc,fast,fastsav
      external dsanwx
      logical first
      data first/.true./,fastsav/0/
      save first,fastsav

c----------------------------------------------------------------------

      oh2=0.0d0
      xf=0.0d0
      nfc=0

c...temporarily change width of h1 higgs boson
      h1width=width(kh1)
      width(kh1)=max(width(kh1),0.1d0) ! je, dec 11, 1998

      if (first) then
         call dsrddof150
         first=.false.
      endif

      if (fast.eq.1) then       ! fast calculation
         dwopt=.true.
         waccd=0.05d0
         dpminr=5.0d-4
         dpthr=2.5d-3
         wdiffr=0.5d0
         wdifft=0.1d0
         mcofr=1.5d0
         brmin=1.0d-4           ! 1.0d-3 doesn't make a big speed difference
      else
         mcofr=2.1d0
      endif

      if (fast.eq.0.and.fastsav.eq.1) then
         write (*,*) 'error in dsrdomega:',
     &    ' can''t go back to fast=0 when fast=1 had been used.'
c        write(*,*) 'rewrite dsrdomega if this is desired.'
         stop
      endif

      fastsav=fast
c--------------------------------------initialize relic density routines
      rdprt = prtlevel

      ncoann=1
      mcoann(1)=mass(kn(1))
      dof(1)=2.0d0
      if (omtype.eq.1.or.omtype.eq.2) then
         mcoann(2)=mass(kn(2))
         mcoann(3)=mass(kn(3))
         mcoann(4)=mass(kn(4))
         dof(2)=2.0d0
         dof(3)=2.0d0
         dof(4)=2.0d0
         if (mcoann(2)/mcoann(1).le.mcofr) ncoann=2
         if (mcoann(3)/mcoann(1).le.mcofr) ncoann=3
         if (mcoann(4)/mcoann(1).le.mcofr) ncoann=4
         if (mass(kcha(1))/mcoann(1).le.mcofr) then
            ncoann=ncoann+1
            mcoann(ncoann)=mass(kcha(1))
            dof(ncoann)=4.0d0
         endif
         if (mass(kcha(2))/mcoann(1).le.mcofr) then
            ncoann=ncoann+1
            mcoann(ncoann)=mass(kcha(2))
            dof(ncoann)=4.0d0
         endif
      endif

c...add resonances and thresholds for new call
      call dsrdres(ncoann,mcoann,nrs,rm,rw)
      call dsrdthr(ncoann,mcoann,nt,tm)

      rdtag=idtag

      if ((omtype.eq.0).or.(omtype.eq.1).or.((omtype.eq.2.).and.
     &     (ncoann.gt.1))) then
         if (omtype.eq.0) then
            coann = 0
         else
            coann = 1
         endif
         if (mass(kn(1)).lt.mass(kcha(2))) then ! delete if stat. later
            call dsrdens(dsanwx,ncoann,mcoann,dof,nrs,rm,rw,
     &           nt,tm,oh2,tf,ierr,iwar)
            nfc=nr
         else
            oh2=0.0d0
            nfc=0
         endif
      elseif (omtype.eq.3) then ! quick-and-dirty
         coann = 0
         call dsrdqad(dsanwx,mcoann(1),oh2,ierr)
         iwar=0
         nfc=2
      else
         oh2 = 0.0d0
         nfc=0
      endif

c...  the following two lines are added by je for test
      if (prtlevel.eq.-1) then
         call dsrdwres            ! je test
         write (6,*) '============== new point =============='
         call dsrdwrate(6,6,1)
         call dsrdwrate(6,6,2)
         call dsrdwrate(6,6,3)
      endif

      dsrdomega = oh2
      xf=mco(1)/tf

      width(kh1)=h1width        ! change back h1 width

      end

