*****************************************************************************
***   subroutine dsprep calculates different frequently used cross
***   sections, annihilation branching ratios and sets different common
***   block variable when a new model has been defined. this routine
***   hasto be called before the relic density or any rates are
***   calculated and is called from dssusy.
***   
*** author: joakim edsjo  edsjo@physics.berkeley.edu
*** date: 98-02-27
*** modified: 99-02-23 pg, 00-08-09 je
*****************************************************************************

      subroutine dsprep
      implicit none
      include 'dssusy.h'
      include 'dsprep.h'
      include 'dsidtag.h'


c------------------------ functions ------------------------------------

      real*8 dsandwdcosnn

c------------------------ variables ------------------------------------

      integer i,j,kh(4)

c----------------------------------------------- set-up common variables

c...flags for subroutines. with the help of these flags, some cpu
c...consuming calculations are only done once per model.
      newmodelanwx=.true.
      newmodelandwdcosnn=.true.
      newmodelep=.true.
      newmodelntdk=.true.
      dsprepcalled=.true.


      kh(1)=kh1
      kh(2)=kh2
      kh(3)=kh3
      kh(4)=khc

      wtot=dsandwdcosnn(0.0d0,0.0d0,kn(1),kn(1))
      abr(1) = prtial(22)/wtot    ! cc-bar
      abr(2) = prtial(25)/wtot    ! bb-bar
      abr(3) = prtial(24)/wtot    ! tt-bar
      abr(4) = prtial(19)/wtot    ! tau+ tau-
      abr(5) = prtial(13)/wtot    ! w+ w-
      abr(6) = prtial(12)/wtot    ! z0 z0
      abr(7) = prtial(5)/wtot     ! h10 h30
      abr(8) = prtial(8)/wtot     ! z0 h10
      abr(9) = prtial(9)/wtot     ! z0 h20
      abr(10)= prtial(11)/wtot    ! w+ h- / w- h+
      abr(11)= prtial(6)/wtot     ! h20 h30
      abr(12)= prtial(26)/wtot    ! gluon gluon
      abr(13)= prtial(17)/wtot    ! mu+ mu-
      abr(14)= prtial(29)/wtot    ! z gamma
      mh(1) = mass(kh1)
      mh(2) = mass(kh2)
      mh(3) = mass(kh3)
      mh(4) = mass(khc)
      mx = mass(kn(1))
      sigmav = 0.38937966d-27*3.d10*wtot/(2.d0*mx**2) ! in cm^3/s
      do i=1,29
         sigv(i) = prtial(i)/wtot*sigmav
      enddo

      do i=1,15
        do j=1,4
          hdw(i,j)=hdwidth(i,j)/width(kh(j))
c            write(*,*) 'hdw(',i,',',j,') = ',hdw(i,j)
        enddo
      enddo


      end
